/**************************************************************************\
  class Options
  =============

  Options and global functions class
\**************************************************************************/

#ifndef _OPTIONS_H_
#define _OPTIONS_H_

#include <iostream>
#include <vector>
#include <string>
#include <cmath>

class Options
{
private:
    static bool overWrite, round, printPov, normalize, optimize;
    static bool inputIsPCM, inputIsBP, quiet, useEpsilon, dontRemove;
    static bool printMesh2, noMeshUnion, ignoreMesh, subMeshBoundaries;
    static bool extraComp;
    static unsigned roundDec;
    static double epsilon, normalLength, errorTolerance, roundVal;

    static std::vector<std::string> transformations;

public:

    static const unsigned pointsPerLine=4;
    static const unsigned trianglesPerLine=4;

    static unsigned ParseOptions(const std::vector<std::string>& commandLine,
                                 std::istream** inFile);
    static bool ParseTransformations(
        const std::vector<std::string>& commandLine, unsigned index);

    static inline bool Compare(double d1, double d2);
    static inline double Round(double x);

    static inline bool   OverWrite()        { return overWrite; }
    static inline bool   Round()            { return round; }
    static inline bool   PrintPov()         { return printPov; }
    static inline bool   PrintMesh2()       { return printMesh2; }
    static inline bool   Normalize()        { return normalize; }
    static inline bool   Optimize()         { return optimize; }
    static inline bool   InputIsPCM()       { return inputIsPCM; }
    static inline bool   InputIsBP()        { return inputIsBP; }
    static inline bool   Quiet()            { return quiet; }
    static inline bool   DontRemove()       { return dontRemove; }
    static inline bool   NoMeshUnion()      { return noMeshUnion; }
    static inline bool   IgnoreMesh()       { return ignoreMesh; }
    static inline bool   SubMeshBoundaries(){ return subMeshBoundaries; }
    static inline bool   ExtraComp()        { return extraComp; }
    static inline double NormalLength()     { return normalLength; }
    static inline double ErrorTolerance()   { return errorTolerance; }

    static inline unsigned TransfCnt() { return transformations.size(); }
    static inline const std::vector<std::string>& Transformations()
    { return transformations; }

    static void PrintSyntax();
    static void PrintMsg(char* format,...);
    static void PrintErrorMsg(char* format,...);

//------------------------------------------------------------------------
};

inline bool Options::Compare(double d1, double d2)
{
    return useEpsilon ? std::fabs(d1-d2)<epsilon : d1==d2;
}

inline double Options::Round(double x)
{
    return std::floor(x*roundVal+.5)/roundVal;
}

#define DOUBLE "%.12g"

#endif

