/**************************************************************************\
  class Point
  ===========
\**************************************************************************/

#include "Point.h"

#include <cmath>

void Point::Rotate(double angle, const Point& p1, const Point& p2,
                   bool translate)
{
    static double oldangle=0;
    static Point oldp1, oldp2;
    static double ox,oy,oz;
    static double mat[3][3];
    double X,Y,Z;

    if(angle != oldangle || p1 != oldp1 || p2 != oldp2)
    {
        double m,s,c,t,txy,txz,tyz,sx,sy,sz;

        oldangle = angle; oldp1 = p1; oldp2 = p2;
        X = p2.x()-p1.x(); Y = p2.y()-p1.y(); Z = p2.z()-p1.z();

        m = X*X+Y*Y+Z*Z;
        t = -(p1.x()*X + p1.y()*Y + p1.z()*Z)/m;
        ox = p1.x()+t*X; oy = p1.y()+t*Y; oz = p1.z()+t*Z;

        m = sqrt(m);
        X /= m; Y /= m; Z /= m;
        s = sin(angle); c = cos(angle); t = 1-c;
        mat[0][0] = t*X*X+c;
        mat[1][1] = t*Y*Y+c;
        mat[2][2] = t*Z*Z+c;
        txy = t*X*Y; sz = s*Z;
        mat[0][1] = txy+sz;
        mat[1][0] = txy-sz;
        txz = t*X*Z; sy = s*Y;
        mat[0][2] = txz-sy;
        mat[2][0] = txz+sy;
        tyz = t*Y*Z; sx = s*X;
        mat[1][2] = tyz+sx;
        mat[2][1] = tyz-sx;
    }

    if(translate)
    {
        xCoord-=ox; yCoord-=oy; zCoord-=oz;
    }

    X = xCoord*mat[0][0] + yCoord*mat[1][0] + zCoord*mat[2][0];
    Y = xCoord*mat[0][1] + yCoord*mat[1][1] + zCoord*mat[2][1];
    Z = xCoord*mat[0][2] + yCoord*mat[1][2] + zCoord*mat[2][2];
    xCoord = X; yCoord = Y; zCoord = Z;

    if(translate)
    {
        xCoord+=ox; yCoord+=oy; zCoord+=oz;
    }
}

