/**************************************************************************\
  class Union
  ===========

  Object-type class which is a composite of Objects
\**************************************************************************/

#ifndef _UNION_H_
#define _UNION_H_

#include "Object.h"

#include <list>

class Union: public Object
{
public:
    Union();
    ~Union();

    const std::string& Name();

    // Writing:
    bool Write(std::ostream&, FileType=PCM);

private:
    typedef std::list<Object*> olist_t;
    olist_t objectList;

// Inherited methods:

    // Reading:
    bool ReadPovray(InputStream&);
    bool ReadPCM(InputStream&);

    // Transformations:
    void RoundValues();
    bool Scale(double, bool verbose=true);
    bool Translate(const Point&, bool verbose=true);
    bool SwapCoords(char axis1, char axis2, bool verbose=true);
    bool Rotate(double angle, const Point&,
                const Point&, bool verbose=true);

    bool Center(bool bottom,
                bool verbose = true,
                const Limits* = 0);
    bool Scale2Bound(double size,
                     bool verbose = true,
                     const Limits* = 0);
    bool PrintBoundaries(const char* caption="Limits",
                         const Limits* = 0);

    void UpdateCurrentLimits();
};

#endif
