/*************************************************************************\
|* Povray triangle mesh compressor v2.1                                  *|
|* ------------------------------------                                  *|
|*                                                                       *|
|* Example how to compile in UNIX:                                       *|
|*                                                                       *|
|* g++ -O3 *.cpp -o meshcomp -lm                                         *|
|*                                                                       *|
\*************************************************************************/

#include "Options.h"
#include "Union.h"

#include <iostream>
#include <fstream>
#include <vector>
#include <string>
#include <cstdlib>
#include <cstdio>
using namespace std;

bool Compress(InputStream& iFile, ostream& oFile)
{
    Union objects;

    Object::FileType type;

    // Read:
    type = Options::InputIsPCM() ? Object::PCM : Object::POVRAY;
    if(!objects.Read(iFile, type))
      return false;

    // Apply transformations:
    objects.ApplyTransformations(Options::Transformations());

    // Write:
    type = Options::PrintMesh2() ? Object::POVRAY2 :
           (Options::PrintPov() ? Object::POVRAY : Object::PCM);
    objects.Write(oFile, type);
    return true;
}


#ifdef WINMESHCOMP
int meshcomp(const vector<string>& commandLine)
{
#else
int main(int argc,char* argv[])
{   vector<string> commandLine(argv, argv+argc);
#endif

    istream* inFile = NULL;
    ostream* outFile = &cout;
    unsigned i;

    if((i=Options::ParseOptions(commandLine, &inFile))==0)
        return EXIT_FAILURE;

    if(commandLine.size()<i+1 && inFile==NULL)
    {
        Options::PrintSyntax();
        return EXIT_SUCCESS;
    }

    /* Check files */
    if(inFile==NULL)
    {
        inFile = new ifstream(commandLine[i++].c_str());
        if(inFile->fail())
        {
            cerr << "Can't open "; perror(commandLine[i-1].c_str());
            return EXIT_FAILURE;
        }
    }

    if(commandLine.size() > i && commandLine[i][0] != '-')
    {
        if(inFile != &cin && commandLine[i-1] == commandLine[i])
        {
            Options::PrintErrorMsg(
                "Source and destination files must be different.\n");
            return EXIT_FAILURE;
        }
        if(!Options::OverWrite())
        {
	    ifstream tmp(commandLine[i].c_str());
            if(!tmp.fail())
            {
                tmp.close();
                if(inFile == &cin)
                {
                    cerr << "File '" << commandLine[i] << "' exists.\n"
                         << "Use the -y flag to overwrite "
                         << "(needed with the -i flag)." << endl;
                    return EXIT_FAILURE;
                }
                cerr << "File '" << commandLine[i] << "' exists. "
                     << "Overwrite? (y/n) ";
                char c;
                cin >> c;
                if(c!='y') return EXIT_SUCCESS;
            }
        }
        outFile = new ofstream(commandLine[i].c_str());
        if(outFile->fail())
        {
            cerr << "Can't create "; perror(commandLine[i].c_str());
            return EXIT_FAILURE;
        }
        i++;
    }

    if(commandLine.size()>i)
    {
        if(!Options::ParseTransformations(commandLine,i))
            return EXIT_FAILURE;
    }

    inFile->precision(12);
    outFile->precision(12);

    InputStream is(*inFile);
    bool Res = Compress(is,*outFile);

// Clean:
    if(inFile!=&cin)
    {
        static_cast<ifstream*>(inFile)->close();
        delete inFile;
    }
    if(outFile!=&cout)
    {
        static_cast<ofstream*>(outFile)->close();
        delete outFile;
    }

    if(!Res) return EXIT_FAILURE;
    return EXIT_SUCCESS;
}

